#ifndef DIALOG_H
#define DIALOG_H

/* 
   dialog.H
   include file for xlogmaster dialog subroutines
*/


/*{{{  Variables  */


/*}}}*/

/*{{{  Subroutines  */
void popup_dialog(gchar*, gchar*, GList*, GList*);
static void dialog_button_pressed(GtkWidget *, gpointer *);



/*}}}*/


#endif /* DIALOG_H */
